# Generated by roxygen2: do not edit by hand

S3method(.DollarNames,errorHandler)
S3method(.DollarNames,remoteDriver)
S3method(.DollarNames,webElement)
S3method(print,rsClientServer)
export(errorHandler)
export(getChromeProfile)
export(getFirefoxProfile)
export(makeFirefoxProfile)
export(remoteDriver)
export(rsDriver)
export(selKeys)
export(webElement)
exportClasses(errorHandler)
exportClasses(remoteDriver)
exportClasses(webElement)
importFrom(caTools,base64decode)
importFrom(caTools,base64encode)
importFrom(httr,build_url)
importFrom(httr,parse_url)
importFrom(methods,new)
importFrom(methods,setRefClass)
importFrom(utils,.DollarNames)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(utils,zip)
importFrom(wdman,selenium)
